﻿using System;
using System.Linq;
using System.IO;
using System.Collections;
using System.Diagnostics;

namespace RenTV
{
    class Program
    {
        struct FileDataInfo
        {
            public String FullName;
            public String FileName;
            public String ShortName;
            public String AiringId;
        }
        private static FileDataInfo FileData;
        private static ArrayList FileList = new ArrayList();

        private static DirectoryInfo DirInfo;
        private static DirectoryInfo TempDirInfo;

        private static String TempDir = "";

#if DEBUG
        private static String DirPath = "X:\\TV Recordings2"; // FOR TESTING
#else
        private static String DirPath = "";
#endif
#if DEBUG
        private static String PhoenixPath = "C:\\PhoenixTools"; // FOR TESTING
#else
        private static String PhoenixPath = "";
#endif
#if DEBUG
        private static Int32 MaxFiles = 1; // FOR TESTING
#else
        private static Int32 MaxFiles;
#endif
        private static Boolean SuppressOutput = false;

        // Argument 1 = the path to the SageTV recordings (optional)
        // Argument 2 = the path to the Phoenix Tools folder (optional)
        // Argument 3 = the number of files to rename (optional)
        // Argument 4 = supress output
        public static void Main(String[] Args)
        {
            try
            {
                if (Args.Length > 0)
                    DirPath = Args[0];

                if (Args.Length > 1)
                    PhoenixPath = Args[1];

                if (Args.Length > 2)
                    MaxFiles = Int32.Parse(Args[2]);
                if (MaxFiles == 0)
                    MaxFiles = Int32.MaxValue;

                if (Args.Length > 3)
                    if (Args[3] == "1")
                        SuppressOutput = true;

                if (DirPath == "")
                    DirPath = ".";
                else if (!DirPath.EndsWith("\\"))
                    DirPath += "\\";

                if (DirPath.StartsWith("\\\\"))
                {
                    Console.WriteLine("*** Cannot use UNC paths to the SageTV recordings folder. ***");
                    Console.ReadLine();
                    return;
                }

                if (PhoenixPath == "")
                    PhoenixPath = ".";
                else if (!PhoenixPath.EndsWith("\\"))
                    PhoenixPath += "\\";

                DirInfo = new DirectoryInfo(DirPath);

                TempDirInfo = DirInfo;
                while (TempDirInfo.Parent != null)
                {
                    TempDir = TempDirInfo.Root + "@RenTV\\";
                    TempDirInfo = new DirectoryInfo(TempDirInfo.Parent.FullName);
                }
                TempDirInfo = new DirectoryInfo(TempDir);

                Directory.CreateDirectory(TempDir);


                // Get all video files
                Int32 FileCount = 0;
                foreach (FileInfo FI in DirInfo.GetFiles())
                {
                    if (FI.Extension == ".mpg" || FI.Extension == ".ts")
                    {
                        FileData = new FileDataInfo();
                        FileData.FullName = FI.FullName;
                        FileData.FileName = FI.Name;
                        FileData.ShortName = FI.Name.Split('.')[0];
                        FileData.AiringId = GetAiringId(FI.Name);
                        if (FileData.AiringId != "")
                        {
                            FileList.Add(FileData);
                            FileCount++;
                            if (FileCount == MaxFiles)
                                break;
                        }
                    }
                }

                // Rename each video file
                foreach (FileDataInfo FD in FileList)
                {
                    // Get all files for the same show
                    ArrayList MoveFiles = new ArrayList();
                    MoveFiles.AddRange(DirInfo.GetFiles(FD.ShortName + "*"));
                    foreach (FileInfo FI in MoveFiles)
                        File.Move(FI.FullName, TempDir + FI.Name);

                    if (!SuppressOutput)
                        Console.WriteLine("Renaming " + FD.FileName);

                    // Run the Phoenix Renamer
                    ProcessStartInfo Command = new ProcessStartInfo("cmd.exe");
                    Command.WindowStyle = ProcessWindowStyle.Hidden;
                    Command.WorkingDirectory = PhoenixPath;
                    Command.Arguments = @"/c phoenix-renamer --renameArtifacts " + TempDir;
                    Process CommandProcess = Process.Start(Command);
                    CommandProcess.WaitForExit();

                    // Move show files back to original directory
                    foreach (FileInfo FI in TempDirInfo.GetFiles())
                    {
                        String NewName = FI.Name;
                        // Add back in the AiringId (if it was removed)
                        if (!FI.Name.Contains(FD.AiringId))
                            NewName = FI.Name.Replace(FI.Extension, "-" + FD.AiringId + "-0" + FI.Extension);
                        else if (FI.Name == FD.FileName)
                            Console.WriteLine("   *** Rename Failed ***");

                        // Move files to original directory
                        try
                        {
                            File.Move(FI.FullName, DirPath + NewName);
                        }
                        catch // duplicate file (part 2)
                        {
                            NewName = FI.Name.Replace(FI.Extension, "-" + FD.AiringId + "-1" + FI.Extension);
                            try
                            {
                                File.Move(FI.FullName, DirPath + NewName);
                            }
                            catch { }
                        }
                    }
                }

                // Delete Temp folder only if empty
                FileInfo[] FileNames = TempDirInfo.GetFiles();
                if (FileNames.Count() == 0)
                    Directory.Delete(TempDirInfo.FullName);

                if (!SuppressOutput)
                {
                    Console.WriteLine("--- Done ---");
                    Console.ReadLine();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("*** Exception thrown. " + ex.Message + " ***");
                Console.ReadLine();
            }
        }

        private static String GetAiringId(String FileName)
        {
            String[] Names = FileName.Split('-');

            if (Names.Length < 3)
                return "";

            String AiringId = Names[Names.Length - 2];

            Int64 Temp;
            if (!Int64.TryParse(AiringId, out Temp))
                return "";

            if (System.Text.RegularExpressions.Regex.IsMatch(Names[1], "[S]\\d\\d[E]\\d\\d"))
                return "";

            return AiringId;
        }
    }
}
